#!/bin/bash
#
#   System hardening script for converting an RHEL3 system to one
#    as certified under EAL3/CAPP
#
#   Copyright (C) 2004  Red Hat, Inc.
#
#   Licenced under the terms of the GNU Public License.  See the
#   file COPYING distributed with this one for a description of
#   your rights and responsibilities.
#

# maintainer note on style:
#  unipolar branches use && or || for readability
#  variables are in CAPS
#  backtick expansions $(use parens syntax)
#  functions are prefixed with a : comment for use with set -x
#  all code is inside a function (for tracking syntax errors w/set -x)
#  function names use StudlyCaps
#  local and readonly are used where appropriate
#  global vars and constants have a leading _
#  to check syntax set _CHECK_SYNTAX_ in env and run with bash -x
#    done this way functions are loaded but Main() is not called


:
: -- %Function Definitions
:


: --AliasSU
#  establish an alias for SU

AliasSU() {

    Title " set an alias for 'su' to discourage misuse"

    # Don't run again if the alias is already present
    grep '^alias su=' /etc/profile >/dev/null && return

    if ShallI "(optional) Add 'su' alias to /etc/profile"; then

	Log "Archiving /etc/profile"
	Archive /etc/profile

	Log  "Adding 'su' alias to /etc/profile"

	echo >> /etc/profile '
# su alias, added for EAL3/CAPP
alias su="echo \"Always use '\''/bin/su -'\'' (see '"$_ECG_FULL"')\""
'

    else
	Log "su alias creation declined"
	# this was optional, no failure
    fi
}


: --Archive
#  cycle out a stale logfile

Archive() {
 
    local DATE=$(date +"%Y%m%d-%H%M")
    local K= NAME= N=

    for K in "$@" ; do
	K=${K%%/}
	if [ -e "$K" ] ; then
	    NAME=$K-$DATE
	    N=0
	    while [ -e "$NAME" ] ; do
		NAME=$K-$DATE-$N
		N=$((N+1))
	    done
	    echo moving "$K" to "$NAME"
	    mv "$K" "$NAME"
	fi
    done
}

:  --IEcho
#   Print message only if interactive

IEcho() {
    [ "$_INTERACTIVE" ] && echo "$@"
}

:  --Ask
#   take user input string, print response to stdout
 
Ask() {
 
    local ANS= DEFAULT="$2"

    [ "$DEFAULT" ] || DEFAULT=y
 
    # non-interactive operation defaults to answering default to all questions
    # no output there.
    if [ "$_INTERACTIVE" ]; then
	echo -n "$1 [$DEFAULT] " >/dev/tty
	read ANS
    else
	Log "   Ask(): not interactive, using default '$DEFAULT'" >/dev/null
	ANS="$DEFAULT"
    fi
 
    [ "$ANS" ] || ANS="$DEFAULT"
    echo "$ANS"
}

:  --AskYN
#   take user input for y/n question
 
AskYN() {
    local ANS=$(Ask "$1 (y/n)" "$2")

    case "$ANS" in
	[yY]*)
	    return 0
	    ;;
	*)  return 1
	    ;;
    esac
}

: -- InstallMissingPackages
#  Search for missing packages on install media

InstallMissingPackages() {
    local PKG_MISSING="$1"
    local RPM_TMP=$(mktemp -d /tmp/rpm.XXXXXX) || Die "Can't make temp dir"
    local SEARCH_PATH

    local NATIVE_ARCH=$(GetNativeArchRe)
    local CROSS_ARCH=$(GetCrossArchRe)

    while [ ! -z "$PKG_MISSING" ]
    do
        local PKG
        local MNT

        LogBlock "Missing packages: $PKG_MISSING"
	IEcho "
Enter one or more path names (shell wildcards permitted, space separated) where
I should look for RPM packages. Put paths containing updates first.

If a path name starts with '/mnt/cdrom', I will try mounting that CD-ROM,
and will unmount it again after searching for packages there.
"

        SEARCH_PATH=$(Ask "RPM path?" "$_RPM_PATH")

	TickOn

	# mount CD-ROMs
        for MNT in $SEARCH_PATH
        do
            # mount CD(s) if not mounted
            expr "$MNT" : "/mnt/cdrom" >/dev/null && {
                mount | grep "$MNT " >/dev/null || {
		    # ignore mount errors
		    mount $MNT || true
		}
            }
        done

	# search for and copy missing packages
        for PKG in $PKG_MISSING
        do
            local RPM
	    local CANDIDATES
	    local PKG_NAME
	    local ARCH_RE
	    local PKG_RE
	    local SUFFIX
	    local RPMNAME
	    	
	    # Looking for pkg/cross or pkg/any ?
	    SUFFIX=$(echo $PKG | sed -n 's/.*\/\(.*\)$/\1/p')
	    # remove suffix from name
	    PKG_NAME=$(echo $PKG | sed 's/\/.*//')

	    case "$SUFFIX" in
	    cross)
		ARCH_RE="$CROSS_ARCH"
		;;
	    any)
		ARCH_RE="($NATIVE_ARCH|$CROSS_ARCH)"
		;;
            *)
		ARCH_RE="$NATIVE_ARCH"
		;;
	    esac

	    # defang regexes in filename (i.e. libstdc++)
	    PKG_RE=$(echo $PKG_NAME | sed 's/\([^-a-zA-Z0-9]\)/\\\1/g')
	    
            for RPM in $(find $SEARCH_PATH | egrep "${PKG_RE}.*\\.${ARCH_RE}\\.rpm\$" 2>/dev/null)
            do
                RPMNAME=$(rpm -q --qf='%{NAME}\n' -p "$RPM" 2>/dev/null)

		# ensure that name matches exactly, don't install
		# PKG-devel or libPKG instead of PKG
                [ ."$RPMNAME" != ."$PKG_NAME" ] && continue

		# Ok, found it. Copy, remove from missing list, and exit loop.
		Log "Copying $RPM"
                cp $RPM $RPM_TMP
		PKG_MISSING=$(echo "$PKG_MISSING" | grep -v "^$PKG\$")
		break
            done
        done

	# unmount CD-ROMs
        for MNT in $SEARCH_PATH
        do
            # unmount CD(s) if mounted
            expr "$MNT" : "/mnt/cdrom" >/dev/null && {
		mount | grep "$MNT " >/dev/null && {
		    # ignore umount errors
		    umount $MNT || true
		}
            }
        done

	TickOff
	
	[ -z "$_INTERACTIVE" -a ! -z "$PKG_MISSING" ] && {
	    Die "Did not find all needed packages. Missing: $PKG_MISSING"
	}
    done

    Log "Found all packages."

    AskYN "Install these packages?" "y" && {
	Log "Installing the packages" 
        rpm -Uvh --replacefiles $RPM_TMP/*.rpm
    }

    rm $RPM_TMP/*.rpm
    rmdir $RPM_TMP
}

: -- GetInstalledPackages
GetInstalledPackages () {
    local NAME ARCH

    local CROSS_ARCH=$(GetCrossArchRe)

    rpm -qa --queryformat='%{NAME} %{ARCH}\n' \
    | while read NAME ARCH
    do
	if echo $ARCH | egrep "^$CROSS_ARCH\$" >/dev/null
	then
	    echo "$NAME/cross"
	else
	    echo "$NAME"
	fi
    done
}

: --IsInList
# Check if package $1 is contained in list of packages (remaining args)
IsInList() {
    local PKG="$1"
    local CANDIDATE=
    local CHOICES=
    local FOUND=
    shift
    
    # PKG/any in list matches both PKG and PKG/cross
    CHOICES=$( ( for F; do echo "$F"; done | sed 's/\/any/\/cross/g'
                 for F; do echo "$F"; done | sed 's/\/any//g'
	       ) | sort | uniq )

    # If looking for PKG/any, ignore all suffixes
    expr $PKG : '.*/any' >/dev/null && {
	PKG=$(echo $PKG | sed 's/\/.*//')
	CHOICES=$( echo "$CHOICES" | sed 's/\/.*//' | uniq)
    }

    for CANDIDATE in $CHOICES
    do
	[ "$PKG" = "$CANDIDATE" ] && {
	    FOUND=y
	    break
	}
    done

    if [ "$FOUND" ]
    then
	return 0
    else
	return 1
    fi
}

: --CheckPackages
#  remove unwanted packages, verify that required packages are
#   installed.

CheckPackages() {

    Title " Configure package selection"
    Log "Analyzing the current package selection"

    TickOn
    
    local ALLIPKGS=$(GetInstalledPackages | sort)
    local REQPKGS=$( StripComments $_PKG_REQ  | sort)
    local OPTPKGS=$( StripComments $_PKG_OPT  | sort)
    local ARCHPKGS=$(StripComments $_PKG_ARCH | sort)
    local TO_RM= PKG= 
    local VERSIONREQ= PACKREQ= ARCHREQ=
    local INST_VER= INST_ARCH=
    local PACKINST= PKG_MISSING=

    LogBlock "installed packages on the system: $ALLIPKGS" >/dev/null

    # remove vsftpd from list on WS, it's not available
    grep WS /etc/redhat-release >/dev/null && {
    	REQPKGS=$(echo "$REQPKGS" | grep -v 'vsftpd')
    }

    [ "$_ADD_OPTIONAL" ] && {
	#LogBlock "Optional packages: $OPTPKGS"

	# add all optional packages except for the binary kernels
	REQPKGS=$(echo "$REQPKGS"; echo "$OPTPKGS" | egrep -v '^(kernel|kernel-smp)$')

	# If arch has no cross-compile variants, remove these from list
	[ $(GetCrossArchRe) = "unavailable" ] && {
		REQPKGS=$(echo "$REQPKGS" | grep -v '/cross')
	}
    }
    
    # check for missing or obsolete packages
    for PACKREQ in $REQPKGS $ARCHPKGS; do
	IsInList "$PACKREQ" "$ALLIPKGS" || {
	    PKG_MISSING="$PKG_MISSING
$PACKREQ"
	}
    done

    # version check for cert critical packages
    #
    while read PACKREQ ARCHREQ VERSIONREQ
    do
	# skip comments and blank lines
	expr "$PACKREQ" : '#' >/dev/null && continue
	expr "$PACKREQ" : '.' >/dev/null || continue

	# We may have more than one of these pkgs on the system,
	# i.e. a ppc64 and a ppc (32-bit) version. Loop over all installed
	# versions, and require reinstall if any one of the does not match.
	for INST_VER in $(rpm -q $PACKREQ --qf='%{VERSION}-%{RELEASE}/%{ARCH}\n' 2>/dev/null | sed 's/ /_/g')
	do
	    # ignore if not installed
	    expr "$INST_VER" : '.*is.not.installed' >/dev/null && continue	

	    # does the conf file specify an architecture this applies to?
	    [ "$ARCHREQ" != '*' ] && {
		INST_ARCH=$(echo "$INST_VER" | sed 's/.*\///')
		# ignore architectures other than installed one
		[ "$INST_ARCH" != "$ARCHREQ" ] && continue
	    }
	    INST_VER=$(echo "$INST_VER" | sed 's/\/.*//')

	    [ ."$VERSIONREQ" != ."$INST_VER" ] && {
		Log "$PACKREQ: $INST_VER, need $VERSIONREQ"

		# Add package to missing list if not already in it
		echo "$PKG_MISSING" | egrep "^$PACKREQ(/.*)?\$" >/dev/null || {
		    PKG_MISSING="$PKG_MISSING
$PACKREQ"
		}
	    }
	done
    done < $_PKG_CERT

    TickOff

    if [ "$PKG_MISSING" ]
    then
    	Log $(echo "$PKG_MISSING" | grep . | wc -l) "package(s) missing"
	if ShallI "Install missing packages from install media"; then
		InstallMissingPackages "$PKG_MISSING"
		# now re-run check
		CheckPackages
		return $?
	else
		Die "Missing packages: $PKG_MISSING"
	fi
    else
    	Log "All required packages are installed."
    fi

    TickOn
    for PACKINST in $ALLIPKGS; do
	IsInList "$PACKINST" "$REQPKGS" "$ARCHPKGS" "$OPTPKGS" || {
	    TO_RM="$TO_RM $PACKINST"
	}
    done
    TickOff

    if [ "$TO_RM" ]
    then
	Log "I want to remove the following packages:"
	Log "$TO_RM"

	if ShallI "Remove these RPMs from the system"; then
	    Log "running: rpm -e --allmatches $TO_RM"

	    TickOn
	    
	    rpm -e --allmatches $TO_RM || {
		Die "
rpm package removal was unsuccessful.
Please do it manually.
You can find the list of packages to be removed in the logfile $_LOGFILE."
	    }

	    TickOff
	else
	    Log "removal of packages has been declined."
	    _FAILURE=1
	fi
    else
	Log "All installed packages are permitted."
    fi
}

: -- CheckAuditTools
CheckAuditTools() {
    local MSG
    MSG=$(test -x /usr/sbin/aucat && aucat -f /dev/null 2>&1) || {
    	Log "You don't appear to have a working version of the audit tools
installed. This is the error message:
	# aucat -f /dev/null
	$MSG
"

        if ShallI "Force removal of currently installed laus package(s)"; then
	   Log "Running rpm -e --allmatches --nodeps --noscripts laus"
	   rpm -e --allmatches --nodeps --noscripts laus
	else
	    Log "removal of packages has been declined."
	    _FAILURE=1
	fi
    }
}

: -- CheckAuditTools2
CheckAuditTools2() {
    local MSG
    MSG=$(aucat -f /dev/null 2>&1) || {
    	Die "Audit tools are still not working. Giving up: $MSG"
    }
}

:  --CloseLog
#  you need an explanation?

CloseLog() {
    [ -e /dev/fd/88 ] && exec 88>&-
}

:  --ConfigureAudit
#   just in case....

ConfigureAudit() {

    Title " Configure audit subsystem"

    if [ -c /dev/audit ]; then
	Log "Audit device file /dev/audit available.  Setting mode to 0600."
	chmod 600 /dev/audit
    else
	if ShallI "Create /dev/audit"; then
	    Log "Creating /dev/audit"
	    mknod -m 600 /dev/audit c 10 224 || {
		Log "Error creating /dev/audit"
		_FAILURE=1
	    }
	else
	    Log "Creation of /dev/audit declined."
	    _FAILURE=1
	fi
    fi

    # chkconfig enable of audit daemon will be done in HardenServices


    if ShallI "Set up $_SYSCTL to configure audit options"; then

	if grep 'dev\.audit' $_SYSCTL > /dev/null; then
	    grep -v 'dev\.audit' $_SYSCTL > $_SYSCTL.new
	    Replace $_SYSCTL removing $_SYSCTL.new
	else
	    Log "Archiving $_SYSCTL"
	    Archive $_SYSCTL
	fi

	echo "
# laus kernel auditing properties
dev.audit.max-messages = 1024
dev.audit.paranoia = 0
dev.audit.attach-all = 0
dev.audit.allow-suspend = 1
" >> $_SYSCTL

    else
	Log "$_SYSCTL audit option setup declined."
	_FAILURE=1
    fi
}


:  --ConfigureFTP
#   disable anonymous ftp

ConfigureFTP() {

    Title " Configure FTP service"

    local V=$_VSFTPD_CONF

    [ -f $V ] || {
	Log "vsftpd not installed?, no configuration changes."
	return 0
    }


    if ShallI "Disable anonymous FTP access"; then

	sed 's/[#]*anonymous_enable=.*/anonymous_enable=NO/' < $V > $V.new
	Replace $V removing $V.new
    else
	Log "Anonymous FTP disable declined."
	_FAILURE=1
    fi
}


:  --ConfigureSSH
#   drop in an evaluated configuration for SSHD

ConfigureSSH() {

    Title " Configure OpenSSH"

    if ShallI "Update sshd configuration"; then
	Replace $_SSHD_DEST/$_SSHD_CONFIG with $_BASE/$_SSHD_CONFIG

    else
	Log "sshd configuration update declined."
	_FAILURE=1
    fi

    # enable sshd by default
    chkconfig --add sshd
    chkconfig --level 345 sshd on
}


:  --ConfigureXinetd
#  ssia
#   this will lock out everything except ftp

ConfigureXinetd() {

    Title " Configure xinetd"

    if ShallI "Update xinetd.conf"; then
	Replace $_XINETD_DEST/$_XINETD_CONF with $_BASE/$_XINETD_CONF

    else
	Log "xinetd configuration declined."
	_FAILURE=1
    fi
}


: --Die
#  log an error and exit

Die() {
    Warn "Fatal: $@"
    TickOff
    exit 1
}


:  --Warn
#  write args to stderr, and log them

Warn() {
    echo "$@" 1>&2
    Log "$@" >/dev/null
}

:  -- GetNativeArchRe
GetNativeArchRe() {
    local NATIVE_ARCH=$(GetArch)

    # determine egrep regexes to match native packages for this arch.
    # "noarch" ones are always treated as native.
    case $NATIVE_ARCH in
    pSeries|iSeries)
    	# special case for kernel pkgs
	NATIVE_ARCH='ppc|ppc64[pi]series'
	;;
    i?86|xSeries)
	# i386 has some pkgs named *.i686.rpm
	NATIVE_ARCH='i.86'
	;;
    esac
    echo "($NATIVE_ARCH|noarch)"
}

:  -- GetCrossArchRe
GetCrossArchRe() {
    local NATIVE_ARCH=$(GetArch)
    local CROSS_ARCH='unavailable'

    # determine egrep regexes to match cross-compile packages for this arch.
    # Watch out, i386 has some pkgs named *.i686.rpm
    case $NATIVE_ARCH in
    pSeries|iSeries)
	CROSS_ARCH='ppc64'
	;;
    x86_64|ia64)
	CROSS_ARCH='i.86'
	;;
    esac

    echo "$CROSS_ARCH"
}

:  --GetArch
#  only need this because iSeries and pSeries are both ppc64

GetArch() {
    local ARCH=$(uname -i)

    if [ "$ARCH" != ppc64 ]; then
	echo $ARCH
	return
    fi

    [ -f /proc/cpuinfo ] || {
	mount /proc || {
	    Die "/proc/cpuinfo not available.  Is /proc mounted?"
	}
    }

    if grep '^machine.*:.*CHRP' /proc/cpuinfo > /dev/null; then
	echo "pSeries"
	return
    elif grep '^machine.*:.*iSeries' /proc/cpuinfo > /dev/null; then
	echo "iSeries"
	return
    fi

    Die "cannot determine if this ppc64 is iSeries or pSeries"

}


: --HardenPamConfig
#  replace sensitive PAM config files

HardenPamConfig() {
    # !@ FIXME -- these should be dropped into place by the RPM

    Title " Configure Pluggable Authentication Modules (PAM)"

    local PAMFILES=$(cd $_BASE/$_PAMDIR;echo *)
    local FILE=

    # Ensure that the opasswd file exists, otherwise password changing breaks
    touch /etc/security/opasswd
    chmod 600 /etc/security/opasswd

    if ShallI "Replace files in $_PAMBASE/$_PAMDIR/"; then
        for FILE in $PAMFILES; do
	    Replace $_PAMBASE/$_PAMDIR/$FILE with $_BASE/$_PAMDIR/$FILE
        done
    else
        Log "replacement of $_PAMBASE/$_PAMDIR/ files declined."
        _FAILURE=1
    fi

    if ShallI "Update $_LOGIN_DEFS_DEST/$_LOGIN_DEFS"; then
	Replace $_LOGIN_DEFS_DEST/$_LOGIN_DEFS with $_BASE/$_LOGIN_DEFS
    else
	Log "replacement of $_LOGIN_DEFS declined."
	_FAILURE=1
    fi
}


: --HardenPermissions
#  set permissions to those required for evaluated configuration

HardenPermissions() {

    Title " Configure SUID/SGID executable permissions"

    local P=$(StripComments $_PERMSFILE)
    local OUTPUT=

    IEcho "All but the following files will have their setuid/setgid bits removed:
"
    IEcho $P | tr '\040' '\012'
    IEcho

    if ShallI "Update setuid/setgid bits";then
	
	set xx $P; shift
	
	P=

	while [ "$1" ]; do P="$P ! -path $1"; shift;done
	
	TickOn
	OUTPUT=$(find / -not -fstype ext3 -prune -o \
	    -type f \( -perm +4000 -o -perm +2000 \) \
	    $P -print0 \
	    | xargs -0r chmod -v -s 2>&1)
	TickOff
	Log "$OUTPUT"
    else
	Log "setuid/setgid bit removal declined."
	_FAILURE=1
    fi
    chgrp wheel /bin/su
    chmod 4710  /bin/su
}


: --HardenServiceLinks
#  clear out runlevel links for all services
#  link in only those called out in _SERVICES

HardenServiceLinks() {

    Title " Configure runlevel links for all services"

    local WERE_ENABLED=$(cd $_SERVICEDIR; ls S* | sed 's/...//' | sort)

    # runlevel link removal:
    if ShallI "Remove all runlevel links from $_SERVICEDIR"; then
	(
	    cd $_SERVICEDIR;
	    rm -f * nosuchfileordirectory_dummy
	)
	Log "all runlevel symlinks removed."
    else
	Log "Removal of runlevel symlinks declined."
	_FAILURE=1
    fi


    local SERVICE= 
    local REQUIRED=$( StripComments $_SVC_REQ | sort)
    local PERMITTED=$(StripComments $_SVC_OPT | sort)
    local S= FOUND= COUNT=0
    
    LogBlock "required services: $REQUIRED"
    
    if ShallI "Make links in $_SERVICEDIR for all required services."; then
	for SERVICE in $REQUIRED; do
	    Log "$_SERVICE_ENABLE $SERVICE $_SERVICE_CMD"
	    [ -f $_SERVICEBASE/$SERVICE ] || {
		Log "Required Service script $SERVICE not in $_SERVICEBASE
Cannot configure service to start"
		_FAILURE=1
		continue
	    }
	    $_SERVICE_ENABLE $SERVICE $_SERVICE_CMD || {
		Log "$_SERVICE_ENABLE failed to enable $SERVICE"
		_FAILURE=1
		continue
	    }
	    COUNT=$((COUNT + 1))
	done
    else
	Log "symlink creation for required services declined"
	_FAILURE=1
    fi

    LogBlock "previously active services: $WERE_ENABLED"
    LogBlock "permitted additional services: $PERMITTED"

    if ShallI "Reactivate those permitted services you had been running"; then
	for SERVICE in $PERMITTED; do

	    FOUND=
	    for S in $WERE_ENABLED; do
		[ $S = $SERVICE ] && {
		    FOUND=1
		    break
		}
	    done
	    [ "$FOUND" ] && {
		Log "$_SERVICE_ENABLE $SERVICE $_SERVICE_CMD"
		$_SERVICE_ENABLE $SERVICE $_SERVICE_CMD || {
		    Log "$_SERVICE_ENABLE failed to enable $SERVICE"
		    continue
		}
		COUNT=$((COUNT + 1))
	    }
	done
    fi
    Log "$COUNT new runlevel symlinks created."
}



: --Log
#  log to descriptor 88 if available
#   if it's not, log to stdout

Log() {
    if [ -e /dev/fd/88 ]; then
	echo "$@" >&88
	[ "$_VERBOSE" ] && echo "$@"
    else
	echo "$@"
    fi
}

Title() {
    [ "$_VERBOSE" ] && {
	echo
	echo "+++" "$@"
	echo
    }
    Log "+++" "$@" >/dev/null
}

:  --FmtEcho
# Print arguments as indented paragraph, skipping blank lines

FmtEcho() {
    echo "$@" | sed '/^ *$/d;' | fmt | sed '2,$s/^/    /' | fmt
}

:  --LogBlock
# Print arguments to stdout as FmtEcho if interactive, but send to 
# log file as a single line
LogBlock() {
    [ "$_VERBOSE" ] && FmtEcho "$@"
    Log $(echo "$@") >/dev/null
}


: --Main
#  the top of the tops
#

Main() {
    local ARGS=$*
    
    # open the log file
    NewLog " --- $(date) script running: $0 args $ARGS"

    # run checks to make sure all of our data files are here
    local K
    readonly _PKG_ARCH=$_BASE/pkgs-$(GetArch).conf    
    readonly _ALLFILES="$_ALLFILES $_PKG_ARCH"

    for K in $_BASE $_BASE/$_PAMDIR; do
	[ -d $K ] || Die "Required directory missing: $K"
    done

    for K in $_ALLFILES; do
	[ -f $K ] || Die "Required file missing: $K"
    done


    # run sanity checks
    [ $(/usr/bin/id -nu) != root ] && {
	Die "This script must be run as root."
    }
    [ ! -f /proc/1/cmdline ] && {
	Die "Please mount /proc"
    }
    mount | grep "type nfs" && {
	Warn "Warning: you have NFS filesystems mounted, which is only allowed with restrictions. See $_ECG."
    }
    mount | grep "type smb" && {
	Warn "Warning: you have SAMBA filesystems mounted, which is only allowed with restrictions. See $_ECG."
    }
    mount | grep "type vfat" && {
	Die "Please unmount all VFAT filesystems.  See $_ECG."
    }
    mount | grep "type ext2" && {
	Die "Please remount your EXT2 filesystems as EXT3.  See $_ECG."
    }
    mount | grep "type autofs" && {
	Die "Please unmount all AUTOFS filesystems.  See $_ECG."
    }
    # !@ there are probably more filesystems we should gripe about

    [ $(rpm -q kernel kernel-smp | grep -v 'not inst' | wc -l) -gt 1 ] && {
    	Warn "Warning: you have more than one kernel package installed, which may cause problems. You should uninstall unused kernels before running the script, see $_ECG."
    }

    # !@ check for other bootable systems in the GRUB config?

    # is dhclient running?
    ps ax | grep -v grep | grep dhclient >/dev/null && {
	Die "Please hard-configure your network (no DHCP)"
    }

    grep 'wheel:.*:.*:root,.' /etc/group >/dev/null || {
	Die "No trusted users. You won't be able to use 'su'. See $_ECG."
    }

    # !@ Perhaps we should disable USB and FireWire?  Modems and ISDN?

    # Are we running on a terminal? If yes, be more verbose.
    if [ -t 0 ]
    then
	_INTERACTIVE=yes
    else
	_INTERACTIVE=
    fi

    _VERBOSE=yes
    _ADD_OPTIONAL=

    # parse the command line
    while [ "$1" ]; do
	case $1 in
	    -h|--help)
		Usage
		exit 0
		;;
	    -i|--interactive)
		_INTERACTIVE=yes
		shift
		;;
	    -a|--automated)
		_INTERACTIVE=
		shift
		;;
	    -v|--verbose)
		_VERBOSE=yes
		shift
		;;
	    --add-optional)
		_ADD_OPTIONAL=yes
		shift
		;;
	    --rpm-path)
		shift
		_RPM_PATH="$1"
		shift
		;;
	    -q|--quiet)
		_VERBOSE=
		shift
		;;
	    *)  Usage
		exit 1
		;;
	esac
    done

    
    [ "$_INTERACTIVE" ] && {
	echo "
You have chosen to run the reconfiguration in interactive mode.
The evaluated configuration requires that *all* the steps are
done. If you want to do this automatically, stop and re-run the
script in noninteractive mode ('-a' option).

The reconfiguration involves removing packages and modifying the
system configuration, which may result in a system that is
not useful to you. For example, the X11 desktop is removed.

Please read the documentation before proceeding.

"
	AskYN "Continue?" "n" || {
	    Die "Reconfiguration declined.  Your system was not modified."
	}
    }

    mount | grep ' / ' | grep "type ext3.*,acl" > /dev/null || {
	UpdateFSTAB || {
	    Die "root filesystem must be ext3 with ACL support on. See $_ECG."
	}
    }

    CheckAuditTools
    CheckPackages
    CheckAuditTools2
    HardenServiceLinks
    HardenPamConfig
    HardenPermissions
    AliasSU
    SecureTTY
    ConfigureSSH
    ConfigureXinetd
    ConfigureFTP
    ConfigureAudit
    SetRunLevel 3
    Reboot

    # if we got here, the reboot was declined
    CloseLog
    TickOff
    Warn "
Your system is NOT in the evaluated configuration.
Please check $_LOGFILE for more details."
    return $_FAILURE
}

: --NewLog
#  log an error message, opening the log if necessary
#   used to initiate logging or to log before we're sure
#   logging has been enabled

NewLog() {

    [ -e /dev/fd/88 ] || {
	# open the logfile
	Archive $_LOGFILE
	exec 88> $_LOGFILE	# May 16, Happy Birthday Wladziu!
    }

    (_VERBOSE=1; Log "$@")
}




: --Reboot
#  reboot the system

Reboot() {

    Title "System reboot"

    [ -x /sbin/zipl ] && {
	ShallI "Run 'zipl' to update the boot loader configuration?" && {
	    zipl
	}
    }
    
    if [ $(GetArch) = "iSeries" ]; then
	# must match certified kernel version
	local VMLINITRD=/boot/vmlinitrd-2.4.21-15.0.2.EL.peterm.eal.3
	[ -f $VMLINITRD ] || Die "Kernel $VMLINITRD missing ?!"

	if ShallI "Copy $VMLINITRD into /proc/iSeries/mf/B/vmlinux and activate"
	then
	    dd if=$VMLINITRD of=/proc/iSeries/mf/B/vmlinux bs=4k
	    echo 'B' > /proc/iSeries/mf/side
	else
	    Log "Kernel activation has been declined."
	    _FAILURE=1
	fi
    fi

    [ "$_BOOTFAIL.$_FAILURE" = . ] && {
	Log "Reconfiguration successful."
	IEcho
	Log "It is now necessary to reboot the system. After the reboot,"
	Log "your system configuration will match the evaluated configuration"
	IEcho
	if ShallI "Reboot the system"; then
	    Log "rebooting the system now. Sleeping for 10 seconds..."
	    CloseLog
	    sync
	    sleep 10
	    shutdown -r now
	    echo "Waiting..."
	    sleep 600
	    exit 0
	else
	    Log "reboot declined.
Please note that the system must be rebooted for
the configuration to be complete."
	    _FAILURE=1
	fi
    }
}


: --Replace
#  drop in an updated version of a file

Replace() {

    local SRC=$3 CMD=$2 DEST=$1
    local MODE= UG=

    [ -f $DEST ] && {
	[ "$CMD" = with ] && cmp $SRC $DEST > /dev/null 2>&1 && {
	    Log "$SRC and $DEST are identical. No change."
	    return 0
	}
	Log "Archiving $DEST"
	MODE=$(stat -c '%a'    $DEST)
	UG=$(  stat -c '%U.%G' $DEST)
	expr $DEST : /etc/init.d >/dev/null || {
            # don't store backups inside /etc/init.d
	    Archive $DEST
	}
    }
    Log "running: cp -p $SRC $DEST"
    cp -p $SRC $DEST
    [ $MODE/$UG != / ] && {
	chmod $MODE $DEST
	chown $UG $DEST
    }
    [ "$CMD" = removing ] && rm -f $SRC
}


: --SecureTTY
#  make sure there are no pty entries in /etc/securetty

SecureTTY() {
    grep '^pts' /etc/securetty 2>&1 > /dev/null && {
	Die "root login over network connections is not permitted.
Please check and correct /etc/securetty"
    }
}

    
: --SetRunLevel
#  modify inittab for a specified runlevel

SetRunLevel() {
    local LEVEL=${1:-3}
    local MODE= UG=
    local I=/etc/inittab
    local CURR_LEVEL=$(sed -n 's/^id:\([0-9]*\):.*/\1/p' $I)

    [ ."$LEVEL" = ."$CURR_LEVEL" ] && return

    if ShallI "Change default runlevel to $LEVEL"; then
	Log "changing default runlevel to $LEVEL"

	sed 's/^id:.:initdefault:/id:'$LEVEL':initdefault:/' < $I > $I.new

	Replace $I removing $I.new
    else
	Log "initdefault change to $LEVEL declined."
	_FAILURE=1
	_BOOTFAIL=1
    fi
}


:  --ShallI
#  allow the user to confirm an action before doing it
 
ShallI() {
    [ "$_INTERACTIVE" ] || Log "*** $1"
    
    AskYN "$1?" "y"
}

: --StripComments
# strip comments from a file, write result on stdout
#  filename in $1 or will use stdin

StripComments() {
    sed 's/#.*//' < $1
}


:  --TickOff
#   turn off heartbeat

TickOff() {
    [ "$_TICKPID" ] && {
	kill -1 $_TICKPID
	wait $_TICKPID
	_TICKPID=
	echo -e " \b"
    }
}

:  --TickOn
#  heartbeat - draw spinning ticker on the screen

TickOn() {
    [ "$_TICKPID" ] && return
    [ "$_INTERACTIVE" ] || return

    (
	trap 'exit 0' 1 2 15

	while :; do
	    echo -ne '/\b'
	    sleep 1
	    echo -ne '-\b'
	    sleep 1
	    echo -ne '\\\b'
	    sleep 1
	    echo -ne '|\b'
	    sleep 1
	done
    ) &
    _TICKPID=$!
}


:  --UpdateFSTAB
#  fix up the ext2/ext3 mounts to have user_xattr,acl
#  remount the offending filesystems

UpdateFSTAB() {

    Title " Configure mount options in /etc/fstab"

    local LINE= CHANGED= FS=
    > /etc/fstab.new
    while read LINE; do
	expr "$LINE" : '#|[ 	][ 	]*$' >/dev/null && {
	    echo "$LINE" >> /etc/fstab.new
	}
	set xx $LINE; shift
	if [ $3 == "ext3" -o $3 == "ext2" ]; then
	    printf "%s\t%s\t%s\t%s\t%s\t%s\n"           \
		$1 $2 ext3 "user_xattr,acl" $5 $6 >> /etc/fstab.new
	    CHANGED="$CHANGED $2"
	else
	    echo "$LINE" >> /etc/fstab.new
	fi
    done < /etc/fstab

    [ "$CHANGED" ] && {
	Replace /etc/fstab removing /etc/fstab.new
	for FS in $CHANGED; do
	    mount $FS -o remount || _FAILURE=1
	done
    }
    [ ! "$_FAILURE" ]
    # don't add code here
    return $?
}


:
: -- %Global Constants
:
readonly _ECG=ECG
readonly _ECG_FULL="Evaluated Configuration Guide"

readonly _LOGFILE=/var/log/eal3-cert.log

# PAM locations
readonly _PAMBASE=/etc
readonly _PAMDIR=pam.d

readonly _BASE=/etc/audit
readonly _PERMSFILE=$_BASE/eal3-perms.conf

readonly _PKG_CERT=$_BASE/pkgs-cert-versions.conf
readonly _PKG_OPT=$_BASE/pkgs-opt.conf
_PKG_ARCH=		# needs code, gets set in Main

# req-pkgs.conf MUST include post-RHEL3U2 errata
#   required for certification

readonly _PKG_REQ=$_BASE/pkgs-req.conf

readonly _SYSCTL=/etc/sysctl.conf
readonly _SERVICEBASE=/etc/rc.d/init.d
readonly _SERVICEDIR=/etc/rc.d/rc3.d
readonly _SERVICE_ENABLE="chkconfig --level 3"
readonly _SERVICE_CMD=on
readonly _SVC_REQ=$_BASE/svc-req.conf
readonly _SVC_OPT=$_BASE/svc-allow.conf

readonly _SSHD_CONFIG=sshd_config
readonly _SSHD_DEST=/etc/ssh
readonly _XINETD_CONF=xinetd.conf
readonly _XINETD_DEST=/etc
readonly _LOGIN_DEFS=login.defs
readonly _LOGIN_DEFS_DEST=/etc
readonly _VSFTPD_CONF=/etc/vsftpd/vsftpd.conf

# this will get the name of the arch-dependent packages file
#  and go readonly in Main()
_ALLFILES="$_PERMSFILE $_PKG_CERT $_PKG_OPT $_PKG_REQ $_SVC_REQ $_SVC_OPT"
_ALLFILES="$_ALLFILES $_BASE/$_SSHD_CONFIG $_BASE/$_XINETD_CONF"
_ALLFILES="$_ALLFILES $_BASE/$_LOGIN_DEFS"


:
: -- %Global Variables
:
# set PATH to include /sbin and /usr/sbin. Be paranoid as well.
PATH=/bin:/usr/bin:/sbin:/usr/sbin
export PATH
#
# these are all default values
#
_BOOTFAIL=
_FAILURE=
_INTERACTIVE=yes
_ADD_OPTIONAL=
_RPM_PATH="/root/rpms /mnt/cd*"
_VERBOSE=
_TICKPID=

: --Usage
#  Print summary of supported options
Usage() {
    echo "Usage: $0 [OPTIONS]
Options:
    -h|--help           Print this message
    -i|--interactive    Prompt for permission before changes (default)
    -a|--automated      No prompts, take all default answers
    -q|--quiet          Be less verbose
                        (see $_LOGFILE for detailed msgs)
    --rpm-path DIRS     Search for missing RPMs in DIRs (default $_RPM_PATH)
			(in order, may contain shell wildcards, must be quoted)
    --add-optional      Treat all optional packages as required
Example:
$0 -a --add-optional --rpm-path '/root/rpms /mnt/cd*'
" >&2
    exit 1
}

[ $_CHECK_SYNTAX_ ] || Main "$@"   # no code after this line!
